@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2005 Robey Pointer <robey@lag.net>
+# Copyright (C) 2003-2006 Robey Pointer <robey@lag.net>
 #
 # This file is part of paramiko.
 #
@@ -21,6 +21,7 @@ L{AuthHandler}
 """
 
 import threading
+import weakref
 
 # this helps freezing utils
 import encodings.utf_8
@@ -28,7 +29,8 @@ import encodings.utf_8
 from paramiko.common import *
 from paramiko import util
 from paramiko.message import Message
-from paramiko.ssh_exception import SSHException, BadAuthenticationType, PartialAuthentication
+from paramiko.ssh_exception import SSHException, AuthenticationException, \
+    BadAuthenticationType, PartialAuthentication
 from paramiko.server import InteractiveQuery
 
 
@@ -38,13 +40,15 @@ class AuthHandler (object):
     """
     
     def __init__(self, transport):
-        self.transport = transport
+        self.transport = weakref.proxy(transport)
         self.username = None
         self.authenticated = False
         self.auth_event = None
         self.auth_method = ''
         self.password = None
         self.private_key = None
+        self.interactive_handler = None
+        self.submethods = None
         # for server mode:
         self.auth_username = None
         self.auth_fail_count = 0
@@ -155,14 +159,14 @@ class AuthHandler (object):
             if not self.transport.is_active():
                 e = self.transport.get_exception()
                 if e is None:
-                    e = SSHException('Authentication failed.')
+                    e = AuthenticationException('Authentication failed.')
                 raise e
             if event.isSet():
                 break
         if not self.is_authenticated():
             e = self.transport.get_exception()
             if e is None:
-                e = SSHException('Authentication failed.')
+                e = AuthenticationException('Authentication failed.')
             # this is horrible.  python Exception isn't yet descended from
             # object, so type(e) won't work. :(
             if issubclass(e.__class__, PartialAuthentication):
@@ -332,7 +336,7 @@ class AuthHandler (object):
         self._send_auth_result(username, method, result)
 
     def _parse_userauth_success(self, m):
-        self.transport._log(INFO, 'Authentication successful!')
+        self.transport._log(INFO, 'Authentication (%s) successful!' % self.auth_method)
         self.authenticated = True
         self.transport._auth_trigger()
         if self.auth_event != None:
@@ -346,11 +350,11 @@ class AuthHandler (object):
             self.transport._log(DEBUG, 'Methods: ' + str(authlist))
             self.transport.saved_exception = PartialAuthentication(authlist)
         elif self.auth_method not in authlist:
-            self.transport._log(INFO, 'Authentication type not permitted.')
+            self.transport._log(INFO, 'Authentication type (%s) not permitted.' % self.auth_method)
             self.transport._log(DEBUG, 'Allowed methods: ' + str(authlist))
             self.transport.saved_exception = BadAuthenticationType('Bad authentication type', authlist)
         else:
-            self.transport._log(INFO, 'Authentication failed.')
+            self.transport._log(INFO, 'Authentication (%s) failed.' % self.auth_method)
         self.authenticated = False
         self.username = None
         if self.auth_event != None:
@@ -407,4 +411,3 @@ class AuthHandler (object):
         MSG_USERAUTH_INFO_RESPONSE: _parse_userauth_info_response,
     }
 
-
